==> cs18-sidecar-0.0.1.dev1531993095.tar.gz: cs18-sidecar-0.0.1.dev1531993095/sidecar/kub_app_instance_identifier_creator.py <==
import time
from logging import Logger
from sidecar.app_instance_identifier import AppInstanceIdentifier
from sidecar.app_instance_identifier_creator import IAppInstanceIdentifierCreator
from sidecar.kub_api_pod_reader import KubApiPodReader
from sidecar.kub_api_service import IKubApiService


class KubAppInstanceIdentifierCreator(IAppInstanceIdentifierCreator):
    def __init__(self, kub_api_service: IKubApiService, logger: Logger) -> None:
        super().__init__(logger=logger)
        self._kub_api_service = kub_api_service

    def create(self, app_name: str, ip_address: str, additional_data: {}) -> AppInstanceIdentifier:
        # TODO: use create_from_instance_id instead of this method where possible because:
        #       a. consistent with aws
        #       b. don't want to have to handle the situation where ip is not available on the pod (maybe can happen
        #          in some pod phases, like 'pending'), which will cause the pod not to be found

        pod_json = self.get_pod_json_by_ip_with_retries(app_name=app_name, ip_address=ip_address)
        return self._create_identifier_for_app_on_pod(app_name, pod_json)

    def get_pod_json_by_ip_with_retries(self, app_name: str, ip_address: str):
        remains = 10
        while True:
            try:
                pod_json = self._kub_api_service.try_get_pod_json_by_ip(ip_address=ip_address)
                if pod_json:
                    return pod_json
                else:
                    if remains == 0:
                        raise Exception("pod for app '{}' with address '{}' not found".format(app_name, ip_address))
                    else:
                        self._logger.info("pod for app '{}' with address '{}' not found ({} remaining attempts)"
                                          .format(app_name, ip_address, remains))
            except Exception as e:
                if remains == 0:
                    raise e
                else:
                    self._logger.exception("error while getting pod json by ip ({REMAINS} remaining attempts)"
                                           .format(REMAINS=remains))

            remains -= 1
            time.sleep(1)

    @staticmethod
    def _create_identifier_for_app_on_pod(app_name: str, pod_json: {}) -> AppInstanceIdentifier:
        # container id is enough as infra identifier because it is supposed to be unique (across space and time),
        # meaning that even if there are multiple instances of the same app on different pods -
        # they should have different container ids
        container_id = KubApiPodReader.safely_get_container_id_for_app(app_name=app_name, pod=pod_json)
        if not container_id:
            raise Exception("container id for app '{APP}' not found on pod '{NAME}'"
                            .format(APP=app_name, NAME=KubApiPodReader.get_pod_name(pod_json)))
        return AppInstanceIdentifier(name=app_name, infra_id=container_id)

    def create_from_instance_id(self, app_name: str, instance_id: str) -> AppInstanceIdentifier:
        # using pod name as instance id and not uid because it can easily be sent from the pod ($HOSTNAME) and
        # because it is more efficient to query by.
        # the advantage of uid is that it is unique over time whereas the pod name isn't, but it should be ok since we
        # only use the pod name to find the pod momentarily and then proceed to identify the instance by container id
        pod_json = self._kub_api_service.try_get_pod_json_by_pod_name(pod_name=instance_id)
        if not pod_json:
            raise Exception("pod with name '{NAME}' not found".format(NAME=instance_id))
        return self._create_identifier_for_app_on_pod(app_name, pod_json)

==> cs18-sidecar-0.0.2.3017.tar.gz: cs18-sidecar-0.0.2.3017/sidecar/kub_app_instance_identifier_creator.py <==
import time
from logging import Logger
from sidecar.app_instance_identifier import AppInstanceIdentifier
from sidecar.app_instance_identifier_creator import IAppInstanceIdentifierCreator
from sidecar.kub_api_pod_reader import KubApiPodReader
from sidecar.kub_api_service import IKubApiService


class KubAppInstanceIdentifierCreator(IAppInstanceIdentifierCreator):
    def __init__(self, kub_api_service: IKubApiService, logger: Logger) -> None:
        super().__init__()
        self._logger = logger
        self._kub_api_service = kub_api_service

    def create(self, app_name: str, ip_address: str, additional_data: {}) -> AppInstanceIdentifier:
        # TODO: use create_from_instance_id instead of this method where possible because:
        #       a. consistent with aws
        #       b. don't want to have to handle the situation where ip is not available on the pod (maybe can happen
        #          in some pod phases, like 'pending'), which will cause the pod not to be found

        pod_json = self.get_pod_json_by_ip_with_retries(ip_address=ip_address)
        return self._create_identifier_for_app_on_pod(app_name, pod_json)

    def get_pod_json_by_ip_with_retries(self, ip_address: str):
        remains = 10
        while True:
            try:
                pod_json = self._kub_api_service.try_get_pod_json_by_ip(ip_address=ip_address)
                if pod_json:
                    return pod_json
                else:
                    if remains == 0:
                        raise Exception("pod with address '{ADDRESS}' not found".format(ADDRESS=ip_address))
                    else:
                        self._logger.info("pod with address '{ADDRESS}' not found ({REMAINS} remaining attempts)"
                                          .format(ADDRESS=ip_address, REMAINS=remains))
            except Exception as e:
                if remains == 0:
                    raise e
                else:
                    self._logger.exception("error while getting pod json by ip ({REMAINS} remaining attempts)"
                                           .format(REMAINS=remains))

            remains -= 1
            time.sleep(1)

    @staticmethod
    def _create_identifier_for_app_on_pod(app_name: str, pod_json: {}) -> AppInstanceIdentifier:
        # container id is enough as infra identifier because it is supposed to be unique (across space and time),
        # meaning that even if there are multiple instances of the same app on different pods -
        # they should have different container ids
        container_id = KubApiPodReader.safely_get_container_id_for_app(app_name=app_name, pod=pod_json)
        if not container_id:
            raise Exception("container id for app '{APP}' not found on pod '{NAME}'"
                            .format(APP=app_name, NAME=KubApiPodReader.get_pod_name(pod_json)))
        return AppInstanceIdentifier(name=app_name, infra_id=container_id)

    def create_from_instance_id(self, app_name: str, instance_id: str) -> AppInstanceIdentifier:
        # using pod name as instance id and not uid because it can easily be sent from the pod ($HOSTNAME) and
        # because it is more efficient to query by.
        # the advantage of uid is that it is unique over time whereas the pod name isn't, but it should be ok since we
        # only use the pod name to find the pod momentarily and then proceed to identify the instance by container id
        pod_json = self._kub_api_service.try_get_pod_json_by_pod_name(pod_name=instance_id)
        if not pod_json:
            raise Exception("pod with name '{NAME}' not found".format(NAME=instance_id))
        return self._create_identifier_for_app_on_pod(app_name, pod_json)
